template_keyword_1: list = [
    ["set_record_enable", [True]],
    ["write", {"write_string": "Hello World"}],
    ["type_keyboard", {"keycode": "return"}],
    ["write", {"write_string": "If you want Windows write upcase string"}],
    ["type_keyboard", {"keycode": "return"}],
    ["write", {"write_string": "you need to press shift first"}],
    ["type_keyboard", {"keycode": "return"}],
    ["press_keyboard_key", {"keycode": "shift"}],
    ["write", {"write_string": "UPCASEE"}],
    ["release_keyboard_key", {"keycode": "shift"}],
    ["type_keyboard", {"keycode": "return"}],
    ["write", {"write_string": "this is example how to use keyword"}],
    ["write", {"write_string": " and add module to executor"}],
    ["type_keyboard", {"keycode": "return"}],
    ["write", {"write_string": "now i will add time module and sleep 3 sec"}],
    ["add_package_to_executor", ["time"]],
    ["time_sleep", [3]],
    ["type_keyboard", {"keycode": "return"}],
    ["write", {"write_string": "also you can use builtin function on executor like print"}],
    ["print", ["Google Bye World"]],
    ["type_keyboard", {"keycode": "return"}],
    ["write", {"write_string": "and i will create an html report for you"}],
    ["generate_html_report"],
    ["type_keyboard", {"keycode": "return"}]
]

template_keyword_2: list = [
    ["set_record_enable", [True]],
    ["write", {"write_string": "in this keyword file we will change the mouse position"}],
    ["type_keyboard", {"keycode": "return"}],
    ["set_mouse_position", [500, 500]],
    ["set_mouse_position", {"x": 100, "y": 100}],
    ["set_mouse_position", [600, 600]],
    ["set_mouse_position", {"x": 100, "y": 100}],
    ["set_mouse_position", [400, 400]],
    ["set_mouse_position", {"x": 100, "y": 100}],
    ["set_mouse_position", [300, 300]],
    ["set_mouse_position", {"x": 100, "y": 100}],
    ["set_mouse_position", [200, 200]],
    ["set_mouse_position", {"x": 100, "y": 100}],
    ["set_mouse_position", [700, 700]],
    ["set_mouse_position", {"x": 100, "y": 100}],
    ["set_mouse_position", [800, 800]],
    ["set_mouse_position", {"x": 100, "y": 100}],
    ["write", {"write_string": "now i will create an json report for you"}],
    ["type_keyboard", {"keycode": "return"}],
    ["generate_json_report"]
]

bad_template_1 = [
    ["set_record_enable", [True]],
    ["add_package_to_executor", ["os"]],
    ["os_system", ["python --version"]],
    ["os_system", ["python -m pip --version"]],
]



import sys
from typing import Tuple

from je_auto_control.utils.exception.exception_tags import osx_import_error
from je_auto_control.utils.exception.exceptions import AutoControlException

if sys.platform not in ["darwin"]:
    raise AutoControlException(osx_import_error)

import time

import Quartz

from je_auto_control.osx.core.utils.osx_vk import osx_mouse_left
from je_auto_control.osx.core.utils.osx_vk import osx_mouse_middle
from je_auto_control.osx.core.utils.osx_vk import osx_mouse_right


def position() -> Tuple[int, int]:
    """
    get mouse current position
    """
    return (Quartz.NSEvent.mouseLocation().x, Quartz.NSEvent.mouseLocation().y)


def mouse_event(event, x: int, y: int, mouse_button: int) -> None:
    """
    :param event which event we want to use
    :param x event x
    :param y event y
    :param mouse_button which mouse button will use event
    """
    curr_event = Quartz.CGEventCreateMouseEvent(None, event, (x, y), mouse_button)
    Quartz.CGEventPost(Quartz.kCGHIDEventTap, curr_event)


def set_position(x: int, y: int) -> None:
    """
    :param x we want to set mouse x position
    :param y we want to set mouse y position
    """
    mouse_event(Quartz.kCGEventMouseMoved, x, y, 0)


def press_mouse(x: int, y: int, mouse_button: int) -> None:
    """
    :param x event x
    :param y event y
    :param mouse_button which mouse button press
    """
    if mouse_button is osx_mouse_left:
        mouse_event(Quartz.kCGEventLeftMouseDown, x, y, Quartz.kCGMouseButtonLeft)
    elif mouse_button is osx_mouse_middle:
        mouse_event(Quartz.kCGEventOtherMouseDown, x, y, Quartz.kCGMouseButtonCenter)
    elif mouse_button is osx_mouse_right:
        mouse_event(Quartz.kCGEventRightMouseDown, x, y, Quartz.kCGMouseButtonRight)


def release_mouse(x: int, y: int, mouse_button: int) -> None:
    """
    :param x event x
    :param y event y
    :param mouse_button which mouse button release
    """
    if mouse_button is osx_mouse_left:
        mouse_event(Quartz.kCGEventLeftMouseUp, x, y, Quartz.kCGMouseButtonLeft)
    elif mouse_button is osx_mouse_middle:
        mouse_event(Quartz.kCGEventOtherMouseUp, x, y, Quartz.kCGMouseButtonCenter)
    elif mouse_button is osx_mouse_right:
        mouse_event(Quartz.kCGEventRightMouseUp, x, y, Quartz.kCGMouseButtonRight)


def click_mouse(x: int, y: int, mouse_button: int) -> None:
    """
    :param x event x
    :param y event y
    :param mouse_button which mouse button click
    """
    if mouse_button is osx_mouse_left:
        press_mouse(x, y, mouse_button)
        time.sleep(.001)
        release_mouse(x, y, mouse_button)
    elif mouse_button is osx_mouse_middle:
        press_mouse(x, y, mouse_button)
        time.sleep(.001)
        release_mouse(x, y, mouse_button)
    elif mouse_button is osx_mouse_right:
        press_mouse(x, y, mouse_button)
        time.sleep(.001)
        release_mouse(x, y, mouse_button)


def scroll(scroll_value: int) -> None:
    """
    :param scroll_value scroll count
    """
    scroll_value = int(scroll_value)
    total = 0
    for do_scroll in range(abs(scroll_value)):
        scroll_event = Quartz.CGEventCreateScrollWheelEvent(
            None,
            0,
            1,
            1 if scroll_value >= 0 else -1
        )
        Quartz.CGEventPost(Quartz.kCGHIDEventTap, scroll_event)
        total = total + do_scroll
    print("Scroll Value:" + total)



import sys

from je_auto_control.utils.exception.exception_tags import osx_import_error
from je_auto_control.utils.exception.exceptions import AutoControlException

if sys.platform not in ["darwin"]:
    raise AutoControlException(osx_import_error)

from Cocoa import *
from Foundation import *
from PyObjCTools import AppHelper

from queue import Queue

record_queue = Queue()

app = NSApplication.sharedApplication()


class AppDelegate(NSObject):
    def applicationDidFinishLaunching_(self, aNotification):
        NSEvent.addGlobalMonitorForEventsMatchingMask_handler_(NSEventMaskKeyDown, keyboard_handler)
        NSEvent.addGlobalMonitorForEventsMatchingMask_handler_(NSEventMaskLeftMouseDown, mouse_left_handler)
        NSEvent.addGlobalMonitorForEventsMatchingMask_handler_(NSEventMaskRightMouseDown, mouse_right_handler)


def mouse_left_handler(event) -> None:
    loc = NSEvent.mouseLocation()
    record_queue.put(("mouse_left", loc.x, loc.y))


def mouse_right_handler(event) -> None:
    loc = NSEvent.mouseLocation()
    record_queue.put(("mouse_right", loc.x, loc.y))


def keyboard_handler(event) -> None:
    if int(event.keyCode()) == 98:
        pass
    else:
        record_queue.put(("type_keyboard", int(hex(event.keyCode()), 16)))
        print(event)


def osx_record() -> None:
    delegate = AppDelegate.alloc().init()
    app.setDelegate_(delegate)
    AppHelper.runEventLoop()


def osx_stop_record() -> Queue:
    return record_queue
